local FOLDER_NAME, private = ...
local _G = _G
local LS = _G.LibStub
local core = LS("AceAddon-3.0"):GetAddon(FOLDER_NAME)
local L = LS("AceLocale-3.0"):GetLocale(FOLDER_NAME)
local strformat, pairs, strsub, collectgarbage = string.format, pairs, string.sub, collectgarbage
local GetItemIcon, ToggleDropDownMenu, HideUIPanel, InterfaceOptionsFrame_OpenToCategory = GetItemIcon, ToggleDropDownMenu, HideUIPanel, InterfaceOptionsFrame_OpenToCategory
local playerName = UnitName("player")
local LDB = LS("LibDataBroker-1.1")
local LTT = LS:GetLibrary("LibQTip-1.0")
local tooltip
local classColors = {}

for class, color in pairs(CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS) do
	classColors[class] = strformat("%2x%2x%2x", color.r * 255, color.g * 255, color.b * 255)
end

function core:createBroker(MODNAME)
	local broker = LDB:NewDataObject("BPM-"..MODNAME, {
		type = "data source",
		text = "",
		label = "BPM-"..MODNAME,
		icon = "Interface\\AddOns\\"..FOLDER_NAME.."\\media\\"..MODNAME,
		OnClick = function(self, button)
			if LTT:IsAcquired("BPM-"..MODNAME.."Tooltip") then
				tooltip:Hide()
				LTT:Release(tooltip)
				tooltip = nil
			end
			if button == "RightButton" then
				local fname, dropdown = nil, nil
				fname = self:GetName()
				dropdown = core:getDropdown(MODNAME)
				private.fname = fname
				private.dropdown = dropdown
				ToggleDropDownMenu(1, nil, dropdown, fname, 0, 0)
			elseif
				button == "MiddleButton" then
				if _G.InterfaceOptionsFrame:IsShown() then
					HideUIPanel(_G.InterfaceOptionsFrame)
				end
				InterfaceOptionsFrame_OpenToCategory(core.optionsFrames.core)
			elseif
				button == "LeftButton" then
				if _G.InterfaceOptionsFrame:IsShown() then
					HideUIPanel(_G.InterfaceOptionsFrame)
				end
				InterfaceOptionsFrame_OpenToCategory(core.optionsFrames[MODNAME])
			end
			collectgarbage("collect")
		end,
		OnEnter = function(self)
			if LTT:IsAcquired("BPM-"..MODNAME.."Tooltip") then
				LTT:Release(tooltip)
				tooltip = nil
			end
			tooltip = LTT:Acquire("BPM-"..MODNAME.."Tooltip", 3)
			tooltip:SetHeaderFont(BPMNameHeader)
			tooltip:SetFont(BPMText)
			tooltip:SetScale(core.db.profile.scale)
			core:drawTooltip(MODNAME)
			tooltip:SmartAnchorTo(self)
			tooltip:SetAutoHideDelay(.1, self)
			tooltip:EnableMouse()
			tooltip:SetScrollStep(core.db.profile.scroll - 25)
			tooltip:UpdateScrolling(core.db.profile.scroll)
			tooltip:Show()
		end,
	})
end

function core:drawTooltip(MODNAME)
	if not private.scanPending then
		core:playerInfo(MODNAME)
	else
		local col = tooltip:GetColumnCount()
		tooltip:SetCell(tooltip:AddLine(), 1, "|cffffff00"..L["Guild Scan Pending"].."|r", BPMText, "CENTER", col)
	end
end

function core:textUpdate(MODNAME)
	local namespace, object = self.db:GetNamespace(MODNAME), nil
	local function objText(k, xp, itemid)
		local color, text
		local name = core:itemName(itemid)
		if core.db.profile.trackerColor then
			color = xp.color[k]
		else
			color = "FFFFFF"
		end
		if not core.db.profile.showShort then
			text = "|cff"..color..name.."|r"
		else
			local shortname = core:shortText(name)
			text = "|cff"..color..shortname.."|r"
		end
		if core.db.profile.showIcons then
			local iconpath = GetItemIcon(itemid)
			local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
			if not core.db.profile.iconsOnly then
				text = icon.." "..text
			else
				text = icon
			end
		else
			text = text
		end
		if text ~= nil and core.db.profile.trackerColor then
			object.text = object.text.." "..text.." "..core:color(core.db.realm.chars[playerName].data[itemid].inv).." "
		elseif text ~= nil then
			object.text = object.text.." "..text.." "..core.db.realm.chars[playerName].data[itemid].inv.." "
		end
	end
	local function xpData(xp)
		for k, itemid in pairs(xp.id) do
			local fixeditem = strsub(itemid, 1, -1)
			if namespace.profiles.Default.tracking[fixeditem] then
				objText(k, xp, itemid)
			end
		end
	end
	local ldbname = "BPM-"..MODNAME
	object = LDB:GetDataObjectByName(ldbname)
	object.text = ""
	if not private.scanPending then
		if private.data[MODNAME].van then
			xpData(private.data[MODNAME].van)
		end
		xpData(private.data[MODNAME].bc)
		xpData(private.data[MODNAME].lk)
		xpData(private.data[MODNAME].cata)
		xpData(private.data[MODNAME].mop)
		xpData(private.data[MODNAME].wod)
		xpData(private.data[MODNAME].leg)
	else
		object.text = " "..("|cffffff00"..L["Guild Scan Pending"].."|r")
	end
end

function core:playerInfo(MODNAME)
	local namespace, xp, xpr, xpg, xpb, xpa = nil, nil, nil, nil, nil, nil
	local col, picol, pbcol, pgcol, hcol, cols, tcol = 0, 0, 0, 0, 3, 1, 0
	local name = "|cff"..classColors[core.db.realm.chars[playerName].class]..strsub(playerName, 1, 20).."|r"
	tooltip:AddLine()
	tooltip:AddSeparator(2, 1, .85, 0, 1)
	tooltip:AddLine()
	tooltip:SetCell(3, 1, L["Item"], BPMColHeader)
	tooltip:SetColumnColor(2, 1, .85, 0, 1)
	tooltip:SetCell(3, 2, "", BPMColHeader)
	tooltip:SetCell(3, 3, L["Inv"], BPMColHeader)
	picol = tooltip:GetColumnCount()
	if self.db.profile.showBank then
		tooltip:AddColumn()
		pbcol = tooltip:GetColumnCount()
		cols = cols + 1
		tooltip:SetCell(3, pbcol, L["Bank"], BPMColHeader)
	end
	if self.db.profile.showGuild then
		tooltip:AddColumn()
		pgcol = tooltip:GetColumnCount()
		cols = cols + 1
		tooltip:SetCell(3, pgcol, L["Guild"], BPMColHeader)
	end
	col = tooltip:GetColumnCount()
	tooltip:SetCell(1, hcol, name, BPMNameHeader, col - 2)
	if core.db.profile.showAlts then core:altInfo() end
	tooltip:AddSeparator(2, 1, .85, 0, 1)
	tcol = tooltip:GetColumnCount()
	if self.db.profile.showTotal then
		tooltip:AddColumn()
		tooltip:SetColumnColor(tcol + 1, 1, .85, 0, 1)
		tooltip:AddColumn()
		tooltip:SetCell(1, tcol + 2, L["All"], BPMTotHeader)
		tooltip:SetCell(3, tcol + 2, L["Total"].." ", BPMColHeader)
	end
	tooltip:AddColumn()
	namespace = core:getNamespace(MODNAME)
	if namespace.profiles.Default.van then
		xp = private.data[MODNAME].van
		xpr, xpg, xpb, xpa = 1, 1, 0, .15
		core:charCounts(xp, xpr, xpg, xpb, xpa, picol, pbcol, pgcol, cols, tcol)
	end
	if namespace.profiles.Default.bc then
		xp = private.data[MODNAME].bc
		xpr, xpg, xpb, xpa = 0, 1, 0, .15
		core:charCounts(xp, xpr, xpg, xpb, xpa, picol, pbcol, pgcol, cols, tcol)
	end
	if namespace.profiles.Default.lk then
		xp = private.data[MODNAME].lk
		xpr, xpg, xpb, xpa = 0, .1, 1, .15
		core:charCounts(xp, xpr, xpg, xpb, xpa, picol, pbcol, pgcol, cols, tcol)
	end
	if namespace.profiles.Default.cata then
		xp = private.data[MODNAME].cata
		xpr, xpg, xpb, xpa = .9, .2, 0, .15
		core:charCounts(xp, xpr, xpg, xpb, xpa, picol, pbcol, pgcol, cols, tcol)
	end
	if namespace.profiles.Default.mop then
		xp = private.data[MODNAME].mop
		xpr, xpg, xpb, xpa = .2, .8, .9, .15
		core:charCounts(xp, xpr, xpg, xpb, xpa, picol, pbcol, pgcol, cols, tcol)
	end
	if namespace.profiles.Default.wod then
		xp = private.data[MODNAME].wod
		xpr, xpg, xpb, xpa = .95, .28, .18, .15
		core:charCounts(xp, xpr, xpg, xpb, xpa, picol, pbcol, pgcol, cols, tcol)
	end
	if namespace.profiles.Default.leg then
		xp = private.data[MODNAME].leg
		xpr, xpg, xpb, xpa = .25, 1, .15, .20
		core:charCounts(xp, xpr, xpg, xpb, xpa, picol, pbcol, pgcol, cols, tcol)
	end
	if tooltip:GetLineCount() == 4 then
		tooltip:Clear()
		for i =1, tooltip:GetColumnCount() do
			tooltip:SetColumnColor(i, nil, nil, nil, 0)
		end
		tooltip:SetCell(tooltip:AddLine(), 1, "|cffff0000"..L["Nothing to display."].."|r", BPMText, "CENTER", tooltip:GetColumnCount())
	end
end

function core:altInfo()
	local itemcols, col, aicol, abcol, agcol, ahcol = 1, 0, 0, 0, 0, 0
	for char, _ in pairs(core.db.realm.chars) do
		if char ~= playerName then
			local name = "|cff"..classColors[core.db.realm.chars[char].class]..strsub(char, 1, 20).."|r"
			tooltip:AddColumn()
			col = tooltip:GetColumnCount()
			tooltip:SetColumnColor(col, 1, .85, 0, 1)
			tooltip:SetCell(3, col, "", BPMColHeader)
			itemcols = itemcols + 2
			tooltip:AddColumn()
			aicol = tooltip:GetColumnCount()
			tooltip:SetCell(3, aicol, L["Inv"], BPMColHeader)
			if self.db.profile.showBank then
				itemcols = itemcols + 1
				tooltip:AddColumn()
				abcol = tooltip:GetColumnCount()
				tooltip:SetCell(3, abcol, L["Bank"], BPMColHeader)
			end
			if self.db.profile.showGuild then
				itemcols = itemcols + 1
				tooltip:AddColumn()
				agcol = tooltip:GetColumnCount()
				tooltip:SetCell(3, agcol, L["Guild"], BPMColHeader)
			end
			ahcol = itemcols + 2
			tooltip:SetCell(1, ahcol, name, BPMNameHeader, col - ahcol + 1)
		end
	end
end

function core:charCounts(xp, xpr, xpg, xpb, xpa, picol, pbcol, pgcol, cols, tcol)
	local total = 0
	cols = cols - 1
	for k, itemid in pairs(xp.id) do
		if core.db.profile.showZero or core:charHasElement(itemid) then
			local line = tooltip:AddLine()
			if core.db.profile.lineColor then tooltip:SetLineColor(line, xpr, xpg, xpb, xpa) end
			tooltip:SetCell(line, 1, core:itemText(xp, k, itemid), BPMText)
			local char = playerName
			tooltip:SetCell(line, picol, core:invCountString(itemid, char), BPMNumbers)
			if self.db.profile.showBank then
				tooltip:SetCell(line, pbcol, core:bankCountString(itemid, char), BPMNumbers)
			end
			if self.db.profile.showGuild then
				tooltip:SetCell(line, pgcol, core:guildCountString(itemid, char), BPMNumbers)
			end
			if core.db.profile.showAlts then
				local x, aicol, abcol, agcol = 0, 0, 0, 0
				for char, _ in pairs(core.db.realm.chars) do
					if char ~= playerName then
						x = x + 1
						aicol = picol + 2 * x + cols * x
						tooltip:SetCell(line, aicol, core:invCountString(itemid, char), BPMNumbers)
						if self.db.profile.showBank then
							abcol = pbcol + 2 * x + cols * x
							tooltip:SetCell(line, abcol, core:bankCountString(itemid, char), BPMNumbers)
						end
						if self.db.profile.showGuild then
							agcol = pgcol + 2 * x + cols * x
							tooltip:SetCell(line, agcol, core:guildCountString(itemid, char), BPMNumbers)
						end
					end
				end
			end
			if core.db.profile.showTotal then
				for char, _ in pairs(core.db.realm.chars) do
					total = core:itemTotal(char, itemid, total)
					tooltip:SetCell(line, tcol + 2, total, BPMNumbers)
				end
			end
		end
	end
end

function core:itemTotal(char, itemid, icount, bcount, gcount, total)
	local icount, bcount, gcount, total = 0, 0, 0, 0
	for char, _ in pairs(core.db.realm.chars) do
		icount = icount + core.db.realm.chars[char].data[itemid].inv
		bcount = bcount + core.db.realm.chars[char].data[itemid].bank
		if self.db.profile.showGuild then
			gcount = gcount + core.db.realm.chars[char].data[itemid].guild
		end
		total = icount + bcount + gcount
	end
	return core:color(total)
end

function core:itemText(xp, k, itemid)
	local color, item
	local name = core:itemName(itemid)
	if core.db.profile.textColor then
		color = xp.color[k]
	else
		color = "FFFFFF"
	end
	if not core.db.profile.tooltipShowShort then
		item = "|cff"..color..name.."|r"
	else
		local shortname = core:shortText(name)
		item = "|cff"..color..shortname.."|r"
	end
	if core.db.profile.tooltipIcons then
		local iconpath = GetItemIcon(itemid)
		local icon = "|T"..iconpath..":12:12:0:0:64:64:4:60:4:60|t"
		if not core.db.profile.tooltipIconsOnly then
			item = icon.." "..item
		else
			item = icon
		end
	else
		item = item
	end
	return item
end
